@@ -1,3 +1,4 @@
+
 /*
  *
  *  Connection Manager
@@ -52,6 +53,7 @@ enum connman_network_error {
 	CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL  = 1,
 	CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL  = 2,
 	CONNMAN_NETWORK_ERROR_INVALID_KEY     = 3,
+	CONNMAN_NETWORK_ERROR_CONNECT_FAIL    = 4,
 };
 
 #define CONNMAN_NETWORK_PRIORITY_LOW      -100
@@ -68,8 +70,6 @@ void connman_network_unref(struct connman_network *network);
 enum connman_network_type connman_network_get_type(struct connman_network *network);
 const char *connman_network_get_identifier(struct connman_network *network);
 
-struct connman_element *connman_network_get_element(
-				struct connman_network *network);
 void connman_network_set_index(struct connman_network *network, int index);
 int connman_network_get_index(struct connman_network *network);
 
@@ -108,8 +108,15 @@ int connman_network_set_name(struct connman_network *network,
 							const char *name);
 int connman_network_set_strength(struct connman_network *network,
 						connman_uint8_t strength);
+connman_uint8_t connman_network_get_strength(struct connman_network *network);
 int connman_network_set_roaming(struct connman_network *network,
 						connman_bool_t roaming);
+int connman_network_set_frequency(struct connman_network *network,
+					connman_uint16_t frequency);
+connman_uint16_t connman_network_get_frequency(struct connman_network *network);
+int connman_network_set_wifi_channel(struct connman_network *network,
+					connman_uint16_t channel);
+connman_uint16_t connman_network_get_wifi_channel(struct connman_network *network);
 
 int connman_network_set_string(struct connman_network *network,
 					const char *key, const char *value);
@@ -119,14 +126,6 @@ int connman_network_set_bool(struct connman_network *network,
 					const char *key, connman_bool_t value);
 connman_bool_t connman_network_get_bool(struct connman_network *network,
 							const char *key);
-int connman_network_set_uint8(struct connman_network *network,
-					const char *key, connman_uint8_t value);
-connman_uint8_t connman_network_get_uint8(struct connman_network *network,
-							const char *key);
-int connman_network_set_uint16(struct connman_network *network,
-					const char *key, connman_uint16_t value);
-connman_uint16_t connman_network_get_uint16(struct connman_network *network,
-							const char *key);
 int connman_network_set_blob(struct connman_network *network,
 			const char *key, const void *data, unsigned int size);
 const void *connman_network_get_blob(struct connman_network *network,
@@ -139,6 +138,9 @@ void connman_network_set_data(struct connman_network *network, void *data);
 
 void connman_network_update(struct connman_network *network);
 
+int connman_network_register(struct connman_network *network);
+void connman_network_unregister(struct connman_network *network);
+
 struct connman_network_driver {
 	const char *name;
 	enum connman_network_type type;
@@ -147,7 +149,6 @@ struct connman_network_driver {
 	void (*remove) (struct connman_network *network);
 	int (*connect) (struct connman_network *network);
 	int (*disconnect) (struct connman_network *network);
-	int (*setup) (struct connman_network *network, const char *key);
 };
 
 int connman_network_driver_register(struct connman_network_driver *driver);
