@@ -57,6 +57,14 @@ void __connman_manager_cleanup(void);
 int __connman_clock_init(void);
 void __connman_clock_cleanup(void);
 
+void __connman_clock_update_timezone(void);
+
+int __connman_timezone_init(void);
+void __connman_timezone_cleanup(void);
+
+char *__connman_timezone_lookup(void);
+int __connman_timezone_change(const char *zone);
+
 int __connman_agent_init(void);
 void __connman_agent_cleanup(void);
 
@@ -72,15 +80,15 @@ int __connman_counter_unregister(const char *owner, const char *path);
 int __connman_counter_init(void);
 void __connman_counter_cleanup(void);
 
-struct connman_service *service;
+struct connman_service;
 
-typedef void (* passphrase_cb_t) (struct connman_service *service,
-				const char *identity, const char *passphrase,
+typedef void (* authentication_cb_t) (struct connman_service *service,
+				const char *identifier, const char *secret,
 				void *user_data);
 typedef void (* report_error_cb_t) (struct connman_service *service,
 				gboolean retry, void *user_data);
 int __connman_agent_request_input(struct connman_service *service,
-				passphrase_cb_t callback, void *user_data);
+				authentication_cb_t callback, void *user_data);
 int __connman_agent_report_error(struct connman_service *service,
 				const char *error,
 				report_error_cb_t callback, void *user_data);
@@ -167,44 +175,9 @@ int __connman_storage_save_service(struct connman_service *service);
 int __connman_storage_load_device(struct connman_device *device);
 int __connman_storage_save_device(struct connman_device *device);
 
-#include <connman/driver.h>
-
-void __connman_driver_rescan(struct connman_driver *driver);
-
-#include <connman/element.h>
-
-int __connman_element_init(const char *device, const char *nodevice);
-void __connman_element_start(void);
-void __connman_element_stop(void);
-void __connman_element_cleanup(void);
-
-void __connman_element_initialize(struct connman_element *element);
-
-typedef void (* element_cb_t) (struct connman_element *element,
-							gpointer user_data);
-
-void __connman_element_foreach(struct connman_element *element,
-				enum connman_element_type type,
-				element_cb_t callback, gpointer user_data);
-void __connman_element_list(struct connman_element *element,
-					enum connman_element_type type,
-							DBusMessageIter *iter);
-
-struct connman_service *__connman_element_get_service(struct connman_element *element);
-struct connman_device *__connman_element_get_device(struct connman_element *element);
-
-struct connman_device *__connman_element_find_device(enum connman_service_type type);
-int __connman_element_request_scan(enum connman_service_type type);
-int __connman_element_enable_technology(enum connman_service_type type);
-int __connman_element_disable_technology(enum connman_service_type type);
-
-gboolean __connman_element_device_isfiltered(const char *devname);
-
 int __connman_detect_init(void);
 void __connman_detect_cleanup(void);
 
-void __connman_element_set_driver(struct connman_element *element);
-
 #include <connman/proxy.h>
 
 int __connman_proxy_init(void);
@@ -255,6 +228,7 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig);
 
 const char *__connman_ipconfig_method2string(enum connman_ipconfig_method method);
+const char *__connman_ipconfig_type2string(enum connman_ipconfig_type type);
 enum connman_ipconfig_method __connman_ipconfig_string2method(const char *method);
 
 void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
@@ -275,6 +249,7 @@ enum connman_ipconfig_method __connman_ipconfig_get_method(
 
 int __connman_ipconfig_address_add(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig);
+int __connman_ipconfig_address_unset(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_gateway_add(struct connman_ipconfig *ipconfig);
 void __connman_ipconfig_gateway_remove(struct connman_ipconfig *ipconfig);
 unsigned char __connman_ipconfig_netmask_prefix_len(const char *netmask);
@@ -312,12 +287,15 @@ int __connman_connection_init(void);
 void __connman_connection_cleanup(void);
 
 int __connman_connection_gateway_add(struct connman_service *service,
-					const char *ipv4_gateway,
-					const char *ipv6_gateway,
+					const char *gateway,
+					enum connman_ipconfig_type type,
 					const char *peer);
-void __connman_connection_gateway_remove(struct connman_service *service);
+void __connman_connection_gateway_remove(struct connman_service *service,
+					enum connman_ipconfig_type type);
 
 gboolean __connman_connection_update_gateway(void);
+void __connman_connection_gateway_activate(struct connman_service *service,
+					enum connman_ipconfig_type type);
 
 int __connman_wpad_init(void);
 void __connman_wpad_cleanup(void);
@@ -353,12 +331,18 @@ connman_bool_t __connman_technology_get_blocked(enum connman_service_type type);
 
 #include <connman/device.h>
 
-int __connman_device_init(void);
+int __connman_device_init(const char *device, const char *nodevice);
 void __connman_device_cleanup(void);
 
 void __connman_device_list(DBusMessageIter *iter, void *user_data);
 
 enum connman_service_type __connman_device_get_service_type(struct connman_device *device);
+struct connman_device *__connman_device_find_device(enum connman_service_type type);
+int __connman_device_request_scan(enum connman_service_type type);
+int __connman_device_enable_technology(enum connman_service_type type);
+int __connman_device_disable_technology(enum connman_service_type type);
+
+connman_bool_t __connman_device_isfiltered(const char *devname);
 
 int __connman_device_get_phyindex(struct connman_device *device);
 void __connman_device_set_phyindex(struct connman_device *device,
@@ -421,6 +405,8 @@ void __connman_config_cleanup(void);
 
 int __connman_config_load_service(GKeyFile *keyfile, const char *group, connman_bool_t persistent);
 int __connman_config_provision_service(struct connman_service *service);
+int __connman_config_provision_service_ident(struct connman_service *service,
+							const char *ident);
 
 #include <connman/profile.h>
 
@@ -448,6 +434,9 @@ void __connman_tethering_update_interface(const char *interface);
 void __connman_tethering_set_enabled(void);
 void __connman_tethering_set_disabled(void);
 
+int __connman_private_network_request(DBusMessage *msg, const char *owner);
+int __connman_private_network_release(const char *path);
+
 #include <connman/provider.h>
 
 void __connman_provider_append_properties(struct connman_provider *provider, DBusMessageIter *iter);
@@ -501,15 +490,15 @@ const char *__connman_service_get_phase2(struct connman_service *service);
 connman_bool_t __connman_service_wps_enabled(struct connman_service *service);
 int __connman_service_set_favorite(struct connman_service *service,
 						connman_bool_t favorite);
-connman_bool_t __connman_service_get_immutable(struct connman_service *service);
 int __connman_service_set_immutable(struct connman_service *service,
 						connman_bool_t immutable);
 
 void __connman_service_set_string(struct connman_service *service,
 					const char *key, const char *value);
-int __connman_service_indicate_state(struct connman_service *service,
+int __connman_service_ipconfig_indicate_state(struct connman_service *service,
 					enum connman_service_state new_state,
 					enum connman_ipconfig_type type);
+
 int __connman_service_indicate_error(struct connman_service *service,
 					enum connman_service_error error);
 int __connman_service_clear_error(struct connman_service *service);
@@ -524,6 +513,8 @@ int __connman_service_create_and_connect(DBusMessage *msg);
 int __connman_service_provision(DBusMessage *msg);
 void __connman_service_auto_connect(void);
 
+void __connman_service_provision_changed(const char *ident);
+
 const char *__connman_service_type2string(enum connman_service_type type);
 
 int __connman_service_nameserver_append(struct connman_service *service,
@@ -563,16 +554,21 @@ int __connman_service_counter_register(const char *counter);
 void __connman_service_counter_unregister(const char *counter);
 
 struct connman_session;
+struct service_entry;
 typedef connman_bool_t (* service_match_cb) (struct connman_session *session,
 					struct connman_service *service);
+typedef struct service_entry* (* create_service_entry_cb) (
+					struct connman_service *service,
+					const char *name,
+					enum connman_service_state state);
 
 GSequence *__connman_service_get_list(struct connman_session *session,
-					service_match_cb service_match);
+				service_match_cb service_match,
+				create_service_entry_cb create_service_entry,
+				GDestroyNotify destroy_service_entry);
 
-connman_bool_t __connman_service_is_connecting(struct connman_service *service);
-connman_bool_t __connman_service_is_connected(struct connman_service *service);
-connman_bool_t __connman_service_is_idle(struct connman_service *service);
-const char *__connman_service_get_name(struct connman_service *service);
+void __connman_service_session_inc(struct connman_service *service);
+connman_bool_t __connman_service_session_dec(struct connman_service *service);
 
 #include <connman/location.h>
 
@@ -670,6 +666,8 @@ int __connman_iptables_commit(const char *table_name);
 
 int __connman_dnsproxy_init(void);
 void __connman_dnsproxy_cleanup(void);
+int __connman_dnsproxy_add_listener(const char *interface);
+void __connman_dnsproxy_remove_listener(const char *interface);
 int __connman_dnsproxy_append(const char *interface, const char *domain, const char *server);
 int __connman_dnsproxy_remove(const char *interface, const char *domain, const char *server);
 void __connman_dnsproxy_flush(void);
