@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <stdio.h>
 #include <net/if.h>
 #include <net/if_arp.h>
@@ -287,6 +288,20 @@ static struct connman_ipaddress *find_ipaddress(struct connman_ipdevice *ipdevic
 	return NULL;
 }
 
+const char *__connman_ipconfig_type2string(enum connman_ipconfig_type type)
+{
+	switch (type) {
+	case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+		return "unknown";
+	case CONNMAN_IPCONFIG_TYPE_IPV4:
+		return "IPv4";
+	case CONNMAN_IPCONFIG_TYPE_IPV6:
+		return "IPv6";
+	}
+
+	return NULL;
+}
+
 static const char *type2str(unsigned short type)
 {
 	switch (type) {
@@ -693,6 +708,7 @@ void __connman_ipconfig_newaddr(int index, int family, const char *label,
 {
 	struct connman_ipdevice *ipdevice;
 	struct connman_ipaddress *ipaddress;
+	enum connman_ipconfig_type type;
 	GList *list;
 
 	DBG("index %d", index);
@@ -714,6 +730,13 @@ void __connman_ipconfig_newaddr(int index, int family, const char *label,
 		return;
 	}
 
+	if (family == AF_INET)
+		type = CONNMAN_IPCONFIG_TYPE_IPV4;
+	else if (family == AF_INET6)
+		type = CONNMAN_IPCONFIG_TYPE_IPV6;
+	else
+		return;
+
 	ipdevice->address_list = g_slist_append(ipdevice->address_list,
 								ipaddress);
 
@@ -740,6 +763,9 @@ void __connman_ipconfig_newaddr(int index, int family, const char *label,
 		if (index != ipconfig->index)
 			continue;
 
+		if (type != ipconfig->type)
+			continue;
+
 		if (ipconfig->ops == NULL)
 			continue;
 
@@ -753,6 +779,7 @@ void __connman_ipconfig_deladdr(int index, int family, const char *label,
 {
 	struct connman_ipdevice *ipdevice;
 	struct connman_ipaddress *ipaddress;
+	enum connman_ipconfig_type type;
 	GList *list;
 
 	DBG("index %d", index);
@@ -765,6 +792,13 @@ void __connman_ipconfig_deladdr(int index, int family, const char *label,
 	if (ipaddress == NULL)
 		return;
 
+	if (family == AF_INET)
+		type = CONNMAN_IPCONFIG_TYPE_IPV4;
+	else if (family == AF_INET6)
+		type = CONNMAN_IPCONFIG_TYPE_IPV6;
+	else
+		return;
+
 	ipdevice->address_list = g_slist_remove(ipdevice->address_list,
 								ipaddress);
 
@@ -787,6 +821,9 @@ void __connman_ipconfig_deladdr(int index, int family, const char *label,
 		if (index != ipconfig->index)
 			continue;
 
+		if (type != ipconfig->type)
+			continue;
+
 		if (ipconfig->ops == NULL)
 			continue;
 
@@ -810,8 +847,10 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 						g_strcmp0(dst, "::") == 0)) {
 		GSList *list;
 		GList *config_list;
+		enum connman_ipconfig_type type;
 
 		if (family == AF_INET6) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV6;
 			g_free(ipdevice->ipv6_gateway);
 			ipdevice->ipv6_gateway = g_strdup(gateway);
 
@@ -822,6 +861,7 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 					g_strdup(gateway);
 			}
 		} else if (family == AF_INET) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV4;
 			g_free(ipdevice->ipv4_gateway);
 			ipdevice->ipv4_gateway = g_strdup(gateway);
 
@@ -848,6 +888,9 @@ void __connman_ipconfig_newroute(int index, int family, unsigned char scope,
 			if (index != ipconfig->index)
 				continue;
 
+			if (type != ipconfig->type)
+				continue;
+
 			if (ipconfig->ops == NULL)
 				continue;
 
@@ -876,8 +919,10 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 						g_strcmp0(dst, "::") == 0)) {
 		GSList *list;
 		GList *config_list;
+		enum connman_ipconfig_type type;
 
 		if (family == AF_INET6) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV6;
 			g_free(ipdevice->ipv6_gateway);
 			ipdevice->ipv6_gateway = NULL;
 
@@ -887,6 +932,7 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 				ipdevice->config_ipv6->system->gateway = NULL;
 			}
 		} else if (family == AF_INET) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV4;
 			g_free(ipdevice->ipv4_gateway);
 			ipdevice->ipv4_gateway = NULL;
 
@@ -912,6 +958,9 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 			if (index != ipconfig->index)
 				continue;
 
+			if (type != ipconfig->type)
+				continue;
+
 			if (ipconfig->ops == NULL)
 				continue;
 
@@ -1083,17 +1132,17 @@ int __connman_ipconfig_gateway_add(struct connman_ipconfig *ipconfig)
 	if (service == NULL)
 		return -EINVAL;
 
-	__connman_connection_gateway_remove(service);
+	__connman_connection_gateway_remove(service, ipconfig->type);
 
-	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
-		return __connman_connection_gateway_add(service, NULL,
-						ipconfig->address->gateway,
-							ipconfig->address->peer);
-	} else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4) {
+	DBG("type %d gw %s peer %s", ipconfig->type,
+		ipconfig->address->gateway, ipconfig->address->peer);
+
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6 ||
+				ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
 		return __connman_connection_gateway_add(service,
 						ipconfig->address->gateway,
-							NULL, ipconfig->address->peer);
-	}
+						ipconfig->type,
+						ipconfig->address->peer);
 
 	return 0;
 }
@@ -1106,7 +1155,7 @@ void __connman_ipconfig_gateway_remove(struct connman_ipconfig *ipconfig)
 
 	service = __connman_service_lookup_from_index(ipconfig->index);
 	if (service != NULL)
-		__connman_connection_gateway_remove(service);
+		__connman_connection_gateway_remove(service, ipconfig->type);
 }
 
 unsigned char __connman_ipconfig_get_prefixlen(struct connman_ipconfig *ipconfig)
@@ -1378,6 +1427,34 @@ int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig)
 
 	DBG("method %d", ipconfig->method);
 
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		err = __connman_ipconfig_address_unset(ipconfig);
+		connman_ipaddress_clear(ipconfig->address);
+
+		return err;
+	}
+
+	return 0;
+}
+
+int __connman_ipconfig_address_unset(struct connman_ipconfig *ipconfig)
+{
+	int err;
+
+	DBG("");
+
+	if (ipconfig == NULL)
+		return 0;
+
+	DBG("method %d", ipconfig->method);
+
 	switch (ipconfig->method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
@@ -1397,8 +1474,6 @@ int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig)
 		else
 			err = -EINVAL;
 
-		connman_ipaddress_clear(ipconfig->address);
-
 		return err;
 	}
 
@@ -1475,6 +1550,14 @@ static void enable_ipv6(struct connman_ipconfig *ipconfig)
 	set_ipv6_state(ipdevice->ifname, TRUE);
 }
 
+void __connman_ipconfig_enable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig == NULL || ipconfig->type != CONNMAN_IPCONFIG_TYPE_IPV6)
+		return;
+
+	enable_ipv6(ipconfig);
+}
+
 void __connman_ipconfig_disable_ipv6(struct connman_ipconfig *ipconfig)
 {
 	if (ipconfig == NULL || ipconfig->type != CONNMAN_IPCONFIG_TYPE_IPV6)
